import { DB } from "../../types/db"
import { managePb404 } from "../../utils/pbTools"
import pbClient from "../pbClient"

/**
 * 从数据库检索一个通知。
 * @param id  要检索的通知的ID。
 * @returns 如果找到，返回解析为通知对象的promise；如果未找到，抛出404错误。
 */
const getOne = (id: string) =>
  managePb404<DB.Notify>(
    async () =>
      await pbClient.collection("notify").getFirstListItem(`build_id="${id}"`)
  )

/**
 * 创建一个新的通知。
 * @param data 新通知的数据。
 * @returns 返回解析为已创建通知对象的promise。
 */
const create = async (data: Partial<DB.Notify>) =>
  await pbClient.collection("notify").create<DB.Notify>(data)

const notify = {
  create,
  getOne,
}

export default notify
